#version 430 core
/*
* Copyright 2018 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

in vec4 ClipSpacePositionOut;
in vec4 PrevClipSpacePositionOut;

out vec4 fragColor;
layout (location = 2) out vec3 MotionVector;

uniform vec3 camera_world;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform sampler2D diffuseTexture;
uniform vec3 color;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL;


vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	return (diffuseF * diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att; 
}

void main() {	
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	
	vec3 texColor = texture(diffuseTexture, vert.uv).rgb;
	fragColor = vec4(texColor, 1);//vec4(texColor * materialCoefficients.x, 1); // ambient
	
	// add directional light contribution
	//fragColor.rgb += phong(n, -dirL.direction, v, dirL.color * texColor, materialCoefficients.y, dirL.color, materialCoefficients.z, specularAlpha, false, vec3(0));
			
	// add point light contribution
	//fragColor.rgb += phong(n, pointL.position - vert.position_world, v, pointL.color * texColor, materialCoefficients.y, pointL.color, materialCoefficients.z, specularAlpha, true, pointL.attenuation);

	//fragColor = vec4(n, 1);
	//fragColor = vec4(vert.uv, 0, 1);
	
	vec3 NCDPos = (ClipSpacePositionOut / ClipSpacePositionOut.w).xyz;
	vec3 PrevNCDPos = (PrevClipSpacePositionOut / PrevClipSpacePositionOut.w).xyz;
	vec4 NCDPosDiff = (ClipSpacePositionOut / ClipSpacePositionOut.w) - (PrevClipSpacePositionOut / PrevClipSpacePositionOut.w);
	vec2 motionVectorCore = NCDPosDiff.xy;
	vec3 testing = NCDPosDiff.xyz;
	MotionVector = testing;
	
}

