#version 430 core

uniform sampler2D image;
uniform sampler2D lensColor;

uniform int numberOfGhosts;
uniform float ghostDispersal;
uniform float haloWidth;

uniform vec3 sunPosition;
uniform mat4 viewProjMatrix;

in vec2 uv;

out vec4 result;

void main() {

	vec4 screenPos = viewProjMatrix * vec4(sunPosition, 1.0);
	
	screenPos /= screenPos.w; // perspective divide
	vec2 uvSun = screenPos.xy;
	vec2 uv2 = -uv + vec2(1.0);
	result = vec4(0.0);
	
	uvSun = uvSun*0.5 + 0.5;
	uvSun.y = 1-uvSun.y;
	uvSun.x = 1-uvSun.x;
	
	if (uvSun.x >= 0.0 && uvSun.y >= 0.0 && uvSun.y <= 0.25 && uvSun.x <= 1.0)
	{
		
		// ghost vector to image center
		vec2 ghostVec = (uvSun - uv2) * ghostDispersal;
		
		float w;
		float d;
		vec2 offset;
		
		for (int i=0;i<numberOfGhosts; ++i)
		{
			offset = fract(uv2 + ghostVec * float(i));
			
			d = distance(uvSun, offset);
			
			w = 1.0 - smoothstep(0.0, 0.75, d); //max(0.0, length(uvSun - offset) / length(vec2(0.5)));
			w = pow(w, 10.0);
			
			result += texture(image, offset) * w;
		}
		
		vec2 haloVec = normalize(ghostVec) * haloWidth;
		offset = fract(uv2 + haloVec);
		d = distance(uvSun, offset);
		w = 1.0 - smoothstep(0.0, 0.75, d); //max(0.0, length(uvSun - fract(uv2 + haloVec)) / length(vec2(0.5)));
		w = pow(w, 5.0);
		result += texture(image, uv2 + haloVec) * w;
		
		result *= texture(lensColor, uv2);
	}
}