#version 430 core
#extension GL_ARB_bindless_texture : enable

layout (local_size_x = 256, local_size_y = 1, local_size_z = 1) in;

layout(std430, binding = 0) buffer position_type_0
{ vec4 Position_Type_In[]; };

layout(std430, binding = 1) buffer velocity_ttl_0
{ vec4 Velocity_TTL_In[]; };

layout(std430, binding = 2) buffer color_radius_0
{ vec4 Color_Rad_In[]; };

layout(std430, binding = 3) buffer position_type_1
{ vec4 Position_Type_Out[]; };

layout(std430, binding = 4) buffer velocity_ttl_1
{ vec4 Velocity_TTL_Out[]; };

layout(std430, binding = 5) buffer color_radius_1
{ vec4 Color_Rad_Out[]; };

layout (binding = 6, offset = 0) uniform atomic_uint Count;

uniform uint LastCount;
uniform uint MaximumCount;
uniform float DeltaT;
const vec3 GRAVITY = vec3(0, -9.81f, 0);

uniform vec3 gEmissionCenter;
uniform float gEmitterSize;
uniform vec3 gEmitterVelocity;
uniform vec3 gEmitterFront;
uniform vec3 gEmitterUp;
uniform float gDeltaTimeMillis;                                                     
uniform float gTime;
uniform float gMaxSize;

uniform int gPatchSize;

//uniform sampler1D gRandomXYZ;       
//uniform sampler1D gRandomLifeTimeSpeed; 

layout(bindless_sampler) uniform sampler1D gRandomLifeTimeSpeed;
layout(bindless_sampler) uniform sampler1D gRandomXYZ;
layout(bindless_sampler) uniform sampler2D gTerrainLookup;

#define PARTICLE_TYPE_LAUNCHER 0.0f                                                 
#define PARTICLE_TYPE_SHELL 1.0f                                                    
#define PARTICLE_TYPE_SECONDARY_SHELL 2.0f 

void addParticleToOutputList(vec3 pos, vec3 vel, vec3 col, float TTL, float type, float rad) {
	// increment the atomic counter and use its previous value:
	uint nr = atomicCounterIncrement(Count);
	if (nr >= MaximumCount) {
		// we are out of memory
		atomicCounterDecrement(Count);
		return; 
	}
	Position_Type_Out[nr] = vec4(pos, type);
	Velocity_TTL_Out[nr] = vec4(vel, TTL);
	Color_Rad_Out[nr] = vec4(col, rad);
}


vec3 RandXYZ( float TexCoord )
{
	vec3 XYZ= texture(gRandomXYZ, TexCoord).xyz;	
	
	float s = gEmitterSize * 2.f;
	
    XYZ = XYZ*s - vec3(s/2, s/2, s/2);
	return XYZ;

}

vec2 RandLifeTimeSpeed( float TexCoord )
{
	return texture(gRandomLifeTimeSpeed, TexCoord).xy;	
}

void emitParticle(int i, float type)
{

	float TexCoord = gTime * i / 1000.f;
	
	vec3 globaldir = -gEmitterFront + gEmitterUp;
	

	vec3 XYZ = RandXYZ( TexCoord );
    vec2 lifeTimeSpeed = RandLifeTimeSpeed( TexCoord );
	
	// range 
	lifeTimeSpeed.x = lifeTimeSpeed.x * 20.f + 2.f;
	lifeTimeSpeed.y = lifeTimeSpeed.y ; //* 5.f + 5.f;

	vec3 pos = XYZ + gEmissionCenter + gEmitterFront * gEmitterSize;
	vec3 vel = normalize(globaldir + XYZ) * lifeTimeSpeed.y;
	vec3 col = vec3(1.0, 1.0, 1.0);
	float LifeTime1 = lifeTimeSpeed.x;
	
	float rad = gMaxSize * LifeTime1;
	float TTL = LifeTime1;
	
	addParticleToOutputList(pos, vel, col, TTL, type, rad);

}                                                                                  


void main() {
	uint idx = gl_GlobalInvocationID.x;
	if(idx >= LastCount) return;
	
	
	float type = Position_Type_In[idx].w;
	vec3 vel = Velocity_TTL_In[idx].xyz;
	vec3 pos = Position_Type_In[idx].xyz;
	vec3 col = Color_Rad_In[idx].xyz;
	float rad = Color_Rad_In[idx].w;
	float TTL = Velocity_TTL_In[idx].w;
	
	if (type == PARTICLE_TYPE_LAUNCHER)
	{
		for (int i=0;i<10;i++)
		{
			emitParticle(i, PARTICLE_TYPE_SHELL);
		}
		
		//addParticleToOutputList(pos, vel, col, TTL, type, rad);
		
	} else {
		vec3 forces = GRAVITY;
		forces += vec3(-5.5f, 0, 0); // add wind or other forces
		vel = vel + gDeltaTimeMillis * forces;
		pos = pos + gDeltaTimeMillis * vel;
		col = col.xyz;
		rad = rad - gDeltaTimeMillis*gMaxSize; // / max(0.1, TTL);
		TTL = TTL - gDeltaTimeMillis;
		
		float terrain_height = texture(gTerrainLookup, vec2(pos.x / gPatchSize, pos.y / gPatchSize)).x;
		
		if (terrain_height > pos.z)
		{
			TTL = 0;
		}
		
		if (TTL > 0) { // particle‘s still alive
			addParticleToOutputList(pos, vel, col, TTL, type, rad);
		} 
		else {
			emitParticle(0, PARTICLE_TYPE_LAUNCHER);
		}
	}
	
	
	
	
	
	
	
}