#version 430 core
#extension GL_ARB_bindless_texture : enable

layout (local_size_x = 256, local_size_y = 1, local_size_z = 1) in;

layout(std430, binding = 0) buffer position_type_0
{ vec4 Position_Type_In[]; };

layout(std430, binding = 1) buffer velocity_ttl_0
{ vec4 Velocity_TTL_In[]; };

layout(std430, binding = 2) buffer color_radius_0
{ vec4 Color_Rad_In[]; };

layout(std430, binding = 3) buffer position_type_1
{ vec4 Position_Type_Out[]; };

layout(std430, binding = 4) buffer velocity_ttl_1
{ vec4 Velocity_TTL_Out[]; };

layout(std430, binding = 5) buffer color_radius_1
{ vec4 Color_Rad_Out[]; };

layout (binding = 6, offset = 0) uniform atomic_uint Count;

uniform uint LastCount;
uniform uint MaximumCount;
uniform float DeltaT;
const vec3 GRAVITY = vec3(0, -9.81f, 0);

uniform vec3 gEmissionCenter;
uniform float gEmitterSize;
uniform vec3 gEmitterVelocity;
uniform vec3 gEmitterFront;
uniform vec3 gEmitterUp;
uniform float gDeltaTimeMillis;                                                     
uniform float gTime;
uniform float gMaxSize;

uniform int gPatchSize;
uniform int gGridCount;
uniform int gMaxHeight;

layout(bindless_sampler) uniform sampler1D gRandomLifeTimeSpeed;
layout(bindless_sampler) uniform sampler1D gRandomXYZ;
layout(bindless_sampler) uniform sampler2D gTerrainLookup;

#define PARTICLE_TYPE_LAUNCHER 0.0f                                                 
#define PARTICLE_TYPE_SHELL 1.0f                                                    
#define PARTICLE_TYPE_SECONDARY_SHELL 2.0f    
#define PARTICLE_TYPE_CYLINDRIC 3.0f    
#define PARTICLE_TYPE_SECONDARY_LAUNCHER 4.0f
#define PARTICLE_SNOWFLAKE 5.0f                                  
#define PI 3.14f 

void addParticleToOutputList(vec3 pos, vec3 vel, vec3 col, float TTL, float type, float rad) {
	// increment the atomic counter and use its previous value:
	uint nr = atomicCounterIncrement(Count);
	if (nr >= MaximumCount) {
		// we are out of memory
		atomicCounterDecrement(Count);
		return; 
	}
	Position_Type_Out[nr] = vec4(pos, type);
	Velocity_TTL_Out[nr] = vec4(vel, TTL);
	Color_Rad_Out[nr] = vec4(col, rad);
}


vec3 RandXYZ( float TexCoord )
{
	vec3 XYZ= texture(gRandomXYZ, TexCoord).xyz;	
	
	float s = gEmitterSize * 2.f;
	
    XYZ = XYZ*s - vec3(s/2, s/2, s/2);
	return XYZ;

}

vec2 RandLifeTimeSpeed( float TexCoord )
{
	return texture(gRandomLifeTimeSpeed, TexCoord).xy;	
}

void emitParticle(float maxrad, float type)
{


	
	uint idx = gl_GlobalInvocationID.x;
	float rad = Color_Rad_In[idx].w;


	float randind = gl_GlobalInvocationID.x / 5000.0f;
	vec3 xyz = texture(gRandomXYZ, randind).xyz;
	
	float r = xyz.y * maxrad;
	float alpha = xyz.x * 2 * PI;
	//float y = maxrad - sqrt(pow(maxrad, 2) - pow(r, 2)) * xyz.y;
	
	// the y value is always same as emission center
	vec3 pos2 = vec3(cos(alpha) * r, gEmissionCenter.y, sin(alpha) * r);//(xyz - vec3(0.5)) * maxrad; //gEmissionCenter + xyz * maxrad;
	
	// 1 ... 2PI
	// a ... alpha --> angle is a * 2PI
	// distance
	// maxrad ... 1
	// r ... a --> r = a * maxrad
	
	
	vec3 vel2 = pos2;
	vec3 col2 = vec3(5*xyz.z, xyz.y, xyz.x);
	// now calculate TTL
	float TTL2 = -distance(pos2, gEmissionCenter);
	float rad2 = rad;
	
	// spawn a shell
	addParticleToOutputList(pos2, vel2, col2, TTL2, type, rad2);

}  

float h_hypslug(float t)
{
	return sqrt(pow(t, 2) + 1);
}	

float x_hypslug(float t, float r)
{
	return r * cos(6*PI*t);
}

float y_hypslug(float t, float r)
{
	return r * sin(6*PI*t);
}

float z_hypslug(float t)
{
	return t;
}

float get_height(float x, float z)
{
	x = x + gPatchSize*gGridCount / 2;
	z = z + gPatchSize*gGridCount / 2;
	// der punkt befindet sich zwischen ...
	int xi1 = int(x / gPatchSize);
	int zi1 = int(z / gPatchSize);
	int xi2 = xi1 + 1;
	int zi2 = zi1 + 1;

	if (xi2 > gGridCount || xi1 < 0 || zi2 > gGridCount || zi1 < 0)
	{
		return -2 * gMaxHeight;
	}

	// prozent der distanz zum oberen eckpunkt
	float distX = (x - float(xi1*gPatchSize)) / gPatchSize;
	float distZ = (z - float(zi1*gPatchSize)) / gPatchSize;

	float heightTop = (1 - distX) * texture(gTerrainLookup, vec2(xi1, zi1)).x + distX * texture(gTerrainLookup, vec2(xi2, zi1)).x;
	float heightBottom = (1 - distX) * texture(gTerrainLookup, vec2(xi1, zi2)).x + distX * texture(gTerrainLookup, vec2(xi2, zi2)).x;
	float height = (1 - distZ) * heightTop + distZ * heightBottom;

	return height;
	
}


void main() {
	uint idx = gl_GlobalInvocationID.x;
	if(idx >= LastCount) return;
	
	
	float type = Position_Type_In[idx].w;
	vec3 vel = Velocity_TTL_In[idx].xyz;
	vec3 pos = Position_Type_In[idx].xyz;
	vec3 col = Color_Rad_In[idx].xyz;
	float rad = Color_Rad_In[idx].w;
	float TTL = Velocity_TTL_In[idx].w;
	
	float offset = col.x*distance(vel.xz, gEmissionCenter.xz);
	if (TTL > offset)
		type = PARTICLE_TYPE_LAUNCHER;
	
	float maxrad = 20.f;
	TTL = TTL + gDeltaTimeMillis/5;//(2*distance(pos, gEmissionCenter));
	float t = TTL ;
	float dist = distance(gEmissionCenter.xz, pos.xz);
	
	if (type == PARTICLE_SNOWFLAKE) {
		vec2 front = normalize(pos.xz-gEmissionCenter.xz);
		float slowdown=0.7;
		pos.xz = pos.xz + front*slowdown;
		pos.y += pow(maxrad*col.y/dist, 2)*slowdown;
		
		addParticleToOutputList(pos, vel, col, TTL, PARTICLE_SNOWFLAKE, rad);
		return;
	}	
	
	if (type == PARTICLE_TYPE_LAUNCHER) {
		// a new particle is spawned.
		emitParticle(maxrad, PARTICLE_TYPE_SECONDARY_SHELL);
		return;
	}
	if (type == PARTICLE_TYPE_SECONDARY_LAUNCHER)
	{		// spawn self
		addParticleToOutputList(pos, vel, col, TTL, PARTICLE_SNOWFLAKE, rad);
		emitParticle(maxrad, PARTICLE_TYPE_SHELL);
		return;
	}
	if (TTL > distance(vel.xz, gEmissionCenter.xz)) // reached the end
		addParticleToOutputList(pos, vel, col, TTL, PARTICLE_TYPE_SECONDARY_LAUNCHER, rad);
	
	
	
	// max distance for particles up to when they start the cylindric movement
	if (type == PARTICLE_TYPE_SHELL || type== PARTICLE_TYPE_SECONDARY_SHELL)
	{
		
		// update movement
		float maxdist = 10.f;
		
		
		
		
		if (maxdist < dist) {
			// draw particle ellipticly to center
			
			vec2 front = normalize(gEmissionCenter.xz-pos.xz);
			
			pos.xz = pos.xz + front;
			
			pos.y =pow(maxrad/dist, 2) + vel.y;
			
			
			addParticleToOutputList(pos, vel, col, TTL, PARTICLE_TYPE_SHELL, rad);


		} else {
			
			// start t must be computed:
			float t = atan(pos.z / pos.x) / (6 * PI);
			col.z = t;
			
			addParticleToOutputList(pos, pos, col, -dist, PARTICLE_TYPE_CYLINDRIC, rad);
		}
		
		if (type == PARTICLE_TYPE_SECONDARY_SHELL)
		{
			// spawn a random particle
			emitParticle(maxrad, PARTICLE_TYPE_SECONDARY_SHELL);
		}
		
		return;
	}
	
	if (type == PARTICLE_TYPE_CYLINDRIC) {
		float r = h_hypslug(t); //min(h_hypslug(t), distance(vel, gEmissionCenter));
		float o = col.z;

		pos.x =  x_hypslug(t, r);
		pos.z =  y_hypslug(t, r);
		pos.y =  col.x*z_hypslug(t) + vel.y + offset;
		
		
		vec2 pos2 = mat2(vec2(cos(o), -sin(o)), vec2(sin(o), cos(o))) * pos.xz;
		
		pos.xz = pos2;
		
		addParticleToOutputList(pos, vel, col, TTL, PARTICLE_TYPE_CYLINDRIC, rad);
	}
	
	
}