#version 430 core
#extension GL_ARB_bindless_texture : enable

layout (local_size_x = 256, local_size_y = 1, local_size_z = 1) in;

layout(std430, binding = 0) buffer position_type_0
{ vec4 Position_Type_In[]; };

layout(std430, binding = 1) buffer velocity_ttl_0
{ vec4 Velocity_TTL_In[]; };

layout(std430, binding = 2) buffer color_radius_0
{ vec4 Color_Rad_In[]; };

layout(std430, binding = 3) buffer position_type_1
{ vec4 Position_Type_Out[]; };

layout(std430, binding = 4) buffer velocity_ttl_1
{ vec4 Velocity_TTL_Out[]; };

layout(std430, binding = 5) buffer color_radius_1
{ vec4 Color_Rad_Out[]; };

layout (binding = 6, offset = 0) uniform atomic_uint Count;

uniform uint LastCount;
uniform uint MaximumCount;
const vec3 GRAVITY = vec3(0, -9.81f, 0);

uniform vec3 gEmissionCenter;
uniform float gEmitterSize;
uniform vec3 gEmitterVelocity;
uniform vec3 gEmitterFront;
uniform vec3 gEmitterUp;
uniform float gDeltaTimeMillis;

uniform bool isfirst;
uniform float max_ttl;
uniform float max_rad;

layout(bindless_sampler) uniform sampler1D gRandomXYZ;

#define PARTICLE_TYPE_TOEMITTER 0.0f                                                 
#define PARTICLE_TYPE_FROMEMITTER 1.0f                                              
#define PARTICLE_TYPE_SHELL 2.0f                         
#define PI 3.14f 

void addParticleToOutputList(vec3 pos, vec3 vel, vec3 col, float TTL, float type, float rad) {
	// increment the atomic counter and use its previous value:
	uint nr = atomicCounterIncrement(Count);
	if (nr >= MaximumCount) {
		// we are out of memory
		atomicCounterDecrement(Count);
		return; 
	}
	Position_Type_Out[nr] = vec4(pos, type);
	Velocity_TTL_Out[nr] = vec4(vel, TTL);
	Color_Rad_Out[nr] = vec4(col, rad);
}


vec3 RandXYZ( float TexCoord )
{
	vec3 XYZ= texture(gRandomXYZ, TexCoord).xyz;	
	
	float s = gEmitterSize * 2.f;
	
    XYZ = XYZ*s - vec3(s/2, s/2, s/2);
	return XYZ;

}

void emitParticle(float maxrad, float type)
{

	uint idx = gl_GlobalInvocationID.x;
	float rad = Color_Rad_In[idx].w;


	float randind = gl_GlobalInvocationID.x / 5000.0f;
	vec3 xyz = texture(gRandomXYZ, randind).xyz;
	
	float r = xyz.x * maxrad;
	float alpha = xyz.z * 2 * PI;
	float y = sqrt(pow(maxrad, 2) - pow(r, 2)) * xyz.y;
	
	// the y value is always same as emission center
	vec3 pos = vec3(cos(alpha) * r, gEmissionCenter.y + y, sin(alpha) * r);//(xyz - vec3(0.5)) * maxrad; //gEmissionCenter + xyz * maxrad;
	
	// 1 ... 2PI
	// a ... alpha --> angle is a * 2PI
	// distance
	// maxrad ... 1
	// r ... a --> r = a * maxrad
	
	vec3 vel = pos;
	
	addParticleToOutputList(pos, pos, vec3(1), max_ttl * xyz.x, type, rad);
	
}

void main() {
	uint idx = gl_GlobalInvocationID.x;
	if(idx >= LastCount) return;
	
	float type = Position_Type_In[idx].w;
	vec3 vel = Velocity_TTL_In[idx].xyz;
	vec3 pos = Position_Type_In[idx].xyz;
	vec3 col = Color_Rad_In[idx].xyz;
	float rad = Color_Rad_In[idx].w;
	float TTL = Velocity_TTL_In[idx].w;
	
	if (isfirst)
	{
	//	float randind = gl_GlobalInvocationID.x / 1000.0f;
	//	vec3 randfl = texture(gRandomXYZ, randind).xyz;
	
		
		float randind = gl_GlobalInvocationID.x / 1000.0f;
		vec3 a = texture(gRandomXYZ, randind).xyz;
	
		addParticleToOutputList(pos, pos, vec3(1.), max_ttl*a.x, PARTICLE_TYPE_TOEMITTER, rad);
		return;
	}
	
	
	if (TTL <= 0)
	{
		// die and spawn new particle at emitter center
		
		if (type == PARTICLE_TYPE_TOEMITTER)
		{
			emitParticle(gEmitterSize, PARTICLE_TYPE_SHELL);
		}
		
		emitParticle(gEmitterSize, PARTICLE_TYPE_FROMEMITTER);
		return;
	}
	TTL = TTL - gDeltaTimeMillis;
	
	
	
	// else move somehow.
	// draw all particles to specific position gEmissionCenter.
	
	vec3 force = GRAVITY;
	
	/*if (type == PARTICLE_TYPE_TOEMITTER)
	{
		force = normalize(gEmissionCenter - pos);	
		
		if (distance(gEmissionCenter, pos) < gEmitterSize)
		{
			// random movement in emitter shell
			type = PARTICLE_TYPE_SHELL;
		}
	}
	else if (type == PARTICLE_TYPE_FROMEMITTER)
	{
		//uint idx = gl_GlobalInvocationID.x;
		//float randind = gl_GlobalInvocationID.x / 1000.0f;
		//force = texture(gRandomXYZ, randind).xyz;
		force += normalize(pos - gEmissionCenter);
		
		if (distance(gEmissionCenter, pos) > gEmitterSize*2)
		{
			type = PARTICLE_TYPE_TOEMITTER;
		}
	}
	else if (type == PARTICLE_TYPE_SHELL)
	{
		// random movement in emitter shell
		uint idx = gl_GlobalInvocationID.x;
		float randind = gl_GlobalInvocationID.x / 2000.0f;
		force = -GRAVITY + normalize(pos - gEmissionCenter) + texture(gRandomXYZ, randind).xyz;
		
		if (distance(gEmissionCenter, pos) > 3*gEmitterSize)
		{
			type = PARTICLE_TYPE_TOEMITTER;
		}
	}*/
	
	
	vel = vel + gDeltaTimeMillis * 3*force;
	pos = pos + gDeltaTimeMillis * vel;
	col = col.xyz;
	
	addParticleToOutputList(pos, vel, col, TTL, type, rad);
	
	
}