#version 430 core

in mat4 modelMatrix1;
in mat4 viewMatrix1;
in mat4 projMatrix1;
in mat4 normalMatrix1;

in vec4 clipSpacePosition1;
in vec3 color1;
in float radius1;
in float s;
in float t;
in float contourConstant1;

// depth awareness
uniform float nearPlane;
uniform float farPlane;

// illumination
uniform vec3 lightVec;
uniform float ambientFactor;
uniform float diffuseFactor;
uniform float specularFactor;
uniform float shininess;
uniform float ambientIntensity;

// bonus: contour
uniform bool contourEnabled;
uniform vec3 contourColor;
uniform float contourDepthFactor;

vec2 ms2(vec3 xyz, float d)
{
	if (xyz.z <= 0)
		return vec2( xyz.x/d, xyz.y/d );
	return vec2( sign(xyz.x) * (1 - (abs(xyz.y)/d)), sign(xyz.y) * ( 1 - (abs(xyz.x) / d)));
}

vec3 invMs2(vec2 uv)
{
	float h = 1 - abs(uv.x) - abs(uv.y);
	if (h >= 0)
		return vec3(uv, h);
	return vec3(sign(uv.x)*(1-abs(uv.y)),sign(uv.x)*(1-abs(uv.y)),h);
}

vec3 addLight(vec3 light, vec3 normal, vec3 viewDir)
{
	vec3 lightDir = normalize(light);
	float spec = 0;
	float intensity = max(dot(normal,lightDir), 0.0);
	// if the vertex is lit compute the specular color
	if (intensity > 0.0) {
		// compute the half vector
		vec3 h = normalize(lightDir + viewDir);  
		// compute the specular term into spec
		float intSpec = max(dot(h,normal), 0.0);
		spec = specularFactor * pow(intSpec,shininess);
	}
	vec3 diffuseColor = intensity * diffuseFactor * vec3(1) + vec3(spec);
	return diffuseColor;
}

void computeColor(float l)
{
	// view direction from camera to world space position
	vec3 viewDir =normalize(vec3(-(inverse(projMatrix1) * clipSpacePosition1))).xyz;
	
	// normal of vertex
	vec4 n =  vec4(s, t, (1 - l), 0.0);
	vec3 normal = normalize((normalMatrix1*n).xyz).xyz;
	
	// illumination
	vec3 ambientColor = vec3(vec3(1) * ambientFactor); // ambient
	vec3 diffuseColor = vec3(0);
	
	diffuseColor += addLight(vec3(0, 0, 1), normal, viewDir);
	diffuseColor += addLight(vec3(0, 1, 0), normal, viewDir);
	diffuseColor += addLight(vec3(1, 0, 0), normal, viewDir);
	/*diffuseColor += addLight(vec3(1, 0, 1), normal, viewDir);
	diffuseColor += addLight(vec3(0, 1, 1), normal, viewDir);
	diffuseColor += addLight(vec3(1, 1, 0), normal, viewDir);
	diffuseColor += addLight(vec3(1, 1, 1), normal, viewDir);
	diffuseColor += addLight(vec3(0, -1, 1), normal, viewDir);
	diffuseColor += addLight(vec3(-1, 1, 0), normal, viewDir);
	diffuseColor += addLight(vec3(1, -1, -1), normal, viewDir);*/
	
	
	gl_FragColor = vec4(max(ambientColor, diffuseColor), 1.0);
}


void main()
{	
	
	float l2 = length(vec2(s * (1+contourConstant1), t*(1+contourConstant1)));
	
	float l = length(vec2(s, t));
	
	// Tiefe nimmt ja nicht linear ab!
	float depth = (clipSpacePosition1.w - sqrt(1-pow(l,2)) * radius1);
	gl_FragDepth = depth / (farPlane - nearPlane);
	
	
	if (l > 1) {
	
		discard;
	}
	
	computeColor(l);
	
	if (contourEnabled && l2 > 1) {
		float w = (l2 - 1) / contourConstant1;
		gl_FragColor = (1-w)*gl_FragColor + (w)*vec4(contourColor,1);
	
		float depth = (clipSpacePosition1.w - sqrt(1-pow(l,2)) * radius1 + contourDepthFactor * (l2 - 1) * radius1);
		gl_FragDepth =  depth / (farPlane - nearPlane);
		
	}

}