#version 430 core
/*
* Copyright 2018 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

in vec4 ClipSpacePositionOut;
in vec4 PrevClipSpacePositionOut;

out vec4 fragColor;
layout (location = 2) out vec3 MotionVector;

uniform vec3 camera_world;

uniform samplerCube cubemap_tex;
uniform vec4 WS_pos_light;
uniform vec2 near_far;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform sampler2D diffuseTexture;
uniform vec3 color;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL;


vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	return (diffuseF * diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att; 
}

float rand(vec2 co){
	float currentvalue = fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);
	return currentvalue; 
}

vec2 grad(vec2 p) {
	vec2 v = vec2(rand(p.xy), rand(p.yx));
    return normalize(v.xy*2.0 - vec2(1.0)); /* remap sampled value to [-1; 1] and normalize */
}


float perlinNoise(vec2 p) {
  /* Calculate lattice points. */
  vec2 p0 = floor(p);
  vec2 p1 = p0 + vec2(1.0, 0.0);
  vec2 p2 = p0 + vec2(0.0, 1.0);
  vec2 p3 = p0 + vec2(1.0, 1.0);
  
  /* Look up gradients at lattice points. */
  vec2 g0 = grad(p0);
  vec2 g1 = grad(p1);
  vec2 g2 = grad(p2);
  vec2 g3 = grad(p3);
    
  float t0 = p.x - p0.x;
  float t1 = p.y - p0.y;

  /* Calculate dot products and interpolate.*/
  float p0p1 = (1.0 - t0) * dot(g0, (p - p0)) + t0 * dot(g1, (p - p1)); /* between upper two lattice points */
  float p2p3 = (1.0 - t0) * dot(g2, (p - p2)) + t0 * dot(g3, (p - p3)); /* between lower two lattice points */
  
  /* max because on Kata's pc there are many black fields */
  return max(0.0,(1.0 - t1) * p0p1 + t1 * p2p3);
}

vec3 generateTexture()
{
	float  randomVal = clamp(rand(vert.position_world.xy*50),0.8,0.9);

	vec2  currentHeigth = vec2(vert.position_world.y,vert.position_world.y);
	float heigthPerlinNoise =perlinNoise( currentHeigth*2)+ perlinNoise( currentHeigth) +perlinNoise( currentHeigth*4);
   float groundPerlinNoise = perlinNoise(vert.position_world.xz/64)+ perlinNoise(vert.position_world.xz/32.0) +perlinNoise(vert.position_world.xz/16.0) + perlinNoise(vert.position_world.xz/8.0);

	
	float snowColor = 0.8;
	float  sumPerlinNoise = (heigthPerlinNoise + groundPerlinNoise*6)/7;
	 sumPerlinNoise = ( sumPerlinNoise +  randomVal)/2;
	

	 sumPerlinNoise =  sumPerlinNoise*0.05+0.865;
	return vec3( sumPerlinNoise, sumPerlinNoise, sumPerlinNoise);
}


bool lies_in_shadow()
{

	// Calculate vector from surface point to light position:
	vec3 cm_lookup_vec = vert.position_world.xyz - WS_pos_light.xyz;
	// Read depth value from cubemap shadow map:
	float d_L = texture(cubemap_tex, cm_lookup_vec).r;
	// Distance between light and fragment:
	float d_S = length(cm_lookup_vec);
	
	//fragColor = vec4(0, d_L, 0, 1.0);
	
	
	// Undo previous [0;1]-mapping. Transform to world space [near,far]:
	d_L = d_L * (near_far.y - near_far.x) + near_far.x;
	// Add a small offset to prevent shadow-acne (Adjust for your scene!)
	const float eps = 0.15;
	
	return d_L + eps < d_S;
}
void compute_color()
{
	
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	
	vec3 texColor = generateTexture();
	//vec3 texColor = texture(diffuseTexture, vert.uv).rgb;
	fragColor = vec4(texColor * materialCoefficients.x, 1); // ambient
	
	fragColor.rgb += phong(n, -dirL.direction, v, dirL.color * texColor, materialCoefficients.y, dirL.color, materialCoefficients.z, specularAlpha, false, vec3(0));

	vec3 NCDPos = (ClipSpacePositionOut / ClipSpacePositionOut.w).xyz;
	vec3 PrevNCDPos = (PrevClipSpacePositionOut / PrevClipSpacePositionOut.w).xyz;
	vec4 NCDPosDiff = (ClipSpacePositionOut / ClipSpacePositionOut.w) - (PrevClipSpacePositionOut / PrevClipSpacePositionOut.w);
	vec2 motionVectorCore = NCDPosDiff.xy;
	vec3 testing = NCDPosDiff.xyz;
	MotionVector = testing;
}

void main() {	
	
	if (lies_in_shadow()) fragColor = vec4(vec3(0), 1);
	else compute_color();
}

